local config = require("modules/utils/config")

local settings = {
	nativeOptions = {},
	nativeSettings = nil,

	strength = 200,
	intensity = 200,
	radius = 60,
	inner = 40,
	outer = 55,
	changedFromPreset = false
}

function settings.setupNative(flashlight)
	local nativeSettings = GetMod("nativeSettings")
	settings.nativeSettings = nativeSettings
	if not nativeSettings then
		print("[Flashlight] Error: NativeSettings lib not found!")
		return
	end

	local cetVer = tonumber((GetVersion():gsub('^v(%d+)%.(%d+)%.(%d+)(.*)', function(major, minor, patch, wip) -- <-- This has been made by psiberx, all credits to him
		return ('%d.%02d%02d%d'):format(major, minor, patch, (wip == '' and 0 or 1))
	end)))

	if cetVer < 1.18 then
		print("[Flashlight] Error: CET version below recommended!")
		return
	end

	nativeSettings.addTab("/flashlight", "Lampe Torche")
	nativeSettings.addSubcategory("/flashlight/gen", "Réglages généraux")
	nativeSettings.addSubcategory("/flashlight/size", "Taille et Distance")
	nativeSettings.addSubcategory("/flashlight/color", "Couleur")

	settings.nativeOptions["brightness"] = nativeSettings.addRangeInt("/flashlight/gen", "Luminosité", "Contrôle la luminosité de la torche", 1, 100, 1, flashlight.settings.brightness, flashlight.defaultSettings.brightness, function(value)
		flashlight.settings.brightness = value
		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["fade"] = nativeSettings.addRangeFloat("/flashlight/gen", "Délai d'allumage/extinction", "Contrôle la durée du fondu lorsque vous allumez/éteignez la lampe torche", 0, 1, 0.01, "%.2f", flashlight.settings.fadeTime, flashlight.defaultSettings.fadeTime, function(value)
		flashlight.settings.fadeTime = value
		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["offset"] = nativeSettings.addRangeFloat("/flashlight/gen", "Décallage avant", "Modifie le décalage vers l'avant de la lampe torche, pour l'empêcher de briller sur le canon de l'arme", 0, 1, 0.05, "%.2f", flashlight.settings.offset, flashlight.defaultSettings.offset, function(value)
		flashlight.settings.offset = value
		config.saveFile("config.json", flashlight.settings)
	end)

	nativeSettings.addSwitch("/flashlight/gen", "Contrôle manette", "Désactive/Active la prise en charge du contrôleur", flashlight.settings.controllerMode, flashlight.defaultSettings.controllerMode, function(state)
		flashlight.settings.controllerMode = state
		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["angle"] = nativeSettings.addRangeInt("/flashlight/size", "Taille", "Modifie la taille du faisceau de la lampe torche", 15, 140, 1, flashlight.settings.angle, flashlight.defaultSettings.angle, function(value)
		flashlight.settings.angle = value
		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["falloff"] = nativeSettings.addRangeInt("/flashlight/size", "Atténuation", "Modifie la force avec laquelle le faisceau de la lampe torche s'estompe sur les bords", 0, 140, 1, flashlight.settings.falloff, flashlight.defaultSettings.falloff, function(value)
		flashlight.settings.falloff = value
		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["range"] = nativeSettings.addRangeInt("/flashlight/size", "Distance", "Modifie jusqu'où la lampe toche éclaire l'environnement", 1, 200, 1, flashlight.settings.range, flashlight.defaultSettings.range, function(value)
		flashlight.settings.range = value
		config.saveFile("config.json", flashlight.settings)
	end)

	local list = {
		[1] = "Personnalisé",
		[2] = "Blanc",
		[3] = "Blanc Chaud",
		[4] = "Rouge",
		[5] = "Vert",
		[6] = "Bleu",
		[7] = "Orange",
		[8] = "Cyan",
		[9] = "Jaune"
	}

	settings.nativeOptions["color"] = nativeSettings.addSelectorString("/flashlight/color", "Préréglage de couleur", "Choisissez parmi l'un des préréglages de couleur ou créez votre propre couleur", list, flashlight.settings.color, flashlight.defaultSettings.color, function(value)
		flashlight.settings.color = value

		if value ~= 1 then
			settings.changedFromPreset = true
			flashlight.settings.red   = flashlight.logic.colors[flashlight.settings.color - 1].Red * 255
			flashlight.settings.green = flashlight.logic.colors[flashlight.settings.color - 1].Green * 255
			flashlight.settings.blue  = flashlight.logic.colors[flashlight.settings.color - 1].Blue * 255

			flashlight.settings.red   = math.floor(flashlight.settings.red)
			flashlight.settings.green = math.floor(flashlight.settings.green)
			flashlight.settings.blue  = math.floor(flashlight.settings.blue)

			settings.nativeSettings.setOption(settings.nativeOptions["red"], flashlight.settings.red)
			settings.nativeSettings.setOption(settings.nativeOptions["green"], flashlight.settings.green)
			settings.nativeSettings.setOption(settings.nativeOptions["blue"], flashlight.settings.blue)

			flashlight.logic.setColorText(flashlight)

			settings.changedFromPreset = false
		end

		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["red"] = nativeSettings.addRangeInt("/flashlight/color", "Rouge", "La quantité de rouge", 0, 255, 1, flashlight.settings.red, flashlight.defaultSettings.red, function(value)
		flashlight.settings.red = value

		if not settings.changedFromPreset and flashlight.settings.color ~= 1 then
			flashlight.settings.color = 1
			settings.nativeSettings.setOption(settings.nativeOptions["color"], 1)
		end

		flashlight.logic.setColorText(flashlight)

		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["green"] = nativeSettings.addRangeInt("/flashlight/color", "Vert", "La quantité de vert", 0, 255, 1, flashlight.settings.green, flashlight.defaultSettings.green, function(value)
		flashlight.settings.green = value

		if not settings.changedFromPreset and flashlight.settings.color ~= 1 then
			flashlight.settings.color = 1
			settings.nativeSettings.setOption(settings.nativeOptions["color"], 1)
		end

		flashlight.logic.setColorText(flashlight)

		config.saveFile("config.json", flashlight.settings)
	end)

	settings.nativeOptions["blue"] = nativeSettings.addRangeInt("/flashlight/color", "Bleu", "La quantité de bleu", 0, 255, 1, flashlight.settings.blue, flashlight.defaultSettings.blue, function(value)
		flashlight.settings.blue = value

		if not settings.changedFromPreset and flashlight.settings.color ~= 1 then
			flashlight.settings.color = 1
			settings.nativeSettings.setOption(settings.nativeOptions["color"], 1)
		end

		flashlight.logic.setColorText(flashlight)

		config.saveFile("config.json", flashlight.settings)
	end)
end

return settings